import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import java.util.ArrayList;

public class MessreiheDurchschnittTest {
  private Messreihe m33244;
  private Messreihe m53532;
  private Messreihe m2344;
  private Messreihe leer;
  private Messreihe mitNullListe;
  private ArrayList<Integer> genutzt;

  private void merkeGenutzteWerte(Messreihe m){
    if (m.getMesswerte() == null){
      return;
    }
    for (int w: m.getMesswerte()) { // in Messreihe keine null-Werte
      this.genutzt.add(w);
    }
  }
  
  private boolean messwerteUnveraendert(Messreihe m){
    if (m.getMesswerte() == null){
      return this.genutzt.size() == 0;
    }
    for(int i = 0; i < this.genutzt.size(); i++) {
      if (this.genutzt.get(i) != m.getMesswerte().get(i)){
        return false;
      }
    }
    return m.getMesswerte().size() == this.genutzt.size();
  }
  
  @BeforeEach
  public void setUp() {
    this.genutzt = new ArrayList<Integer>();
    this.m33244 = new Messreihe("Ort1");
    this.m33244.hinzufuegen(3);
    this.m33244.hinzufuegen(3);
    this.m33244.hinzufuegen(2);
    this.m33244.hinzufuegen(4);
    this.m33244.hinzufuegen(4);
    this.m53532 = new Messreihe("Ort2");
    this.m53532.hinzufuegen(5);
    this.m53532.hinzufuegen(3);
    this.m53532.hinzufuegen(5);
    this.m53532.hinzufuegen(3);
    this.m53532.hinzufuegen(2);
    this.m2344 = new Messreihe("Ort3");
    this.m2344.hinzufuegen(2);
    this.m2344.hinzufuegen(3);
    this.m2344.hinzufuegen(4);
    this.m2344.hinzufuegen(4);
    this.leer = new Messreihe("Ort4");
    this.mitNullListe = new Messreihe("Ort5");
    this.mitNullListe.setMesswerte(null);
  }

  @Test
  public void test01() {
    this.merkeGenutzteWerte(this.m33244);
    Assertions.assertTrue(
        this.m33244.durchschnitt() == 3.2
        , this.m33244 + " hat Durchschnitt 3.2"
            + " berechnet wurde " + this.m33244.durchschnitt());
    Assertions.assertTrue(this.messwerteUnveraendert(this.m33244)
        , "urspruengliche Werte " + this.genutzt 
          + " sollten nicht geaendert werden:" + this.m33244);        
  }

  @Test
  public void test02() {
    this.merkeGenutzteWerte(this.m53532);
    Assertions.assertTrue(
        this.m53532.durchschnitt() == 3.6
        , this.m53532 + " hat Durchschnitt 3.6"
            + " berechnet wurde " + this.m53532.durchschnitt());
    Assertions.assertTrue(this.messwerteUnveraendert(this.m53532)
        , "urspruengliche Werte " + this.genutzt 
          + " sollten nicht geaendert werden:" + this.m53532);        
  }

  @Test
  public void test03() {
    this.merkeGenutzteWerte(this.m2344);
    Assertions.assertTrue(
        this.m2344.durchschnitt() == 3.25
        , this.m2344 + " hat Durchschnitt 3.25"
            + " berechnet wurde " + this.m2344.durchschnitt());
    Assertions.assertTrue(this.messwerteUnveraendert(this.m2344)
        , "urspruengliche Werte " + this.genutzt 
          + " sollten nicht geaendert werden:" + this.m2344);        
  }

  @Test
  public void test04() {
    this.merkeGenutzteWerte(this.leer);
    Assertions.assertTrue(
        this.leer.durchschnitt() == 0
        , this.leer + " hat Durchschnitt 0"
            + " berechnet wurde " + this.leer.durchschnitt());
    Assertions.assertTrue(this.messwerteUnveraendert(this.leer)
        , "urspruengliche Werte " + this.genutzt 
          + " sollten nicht geaendert werden:" + this.leer);        
  }

  @Test
  public void test05() {
    this.merkeGenutzteWerte(this.mitNullListe);
    Assertions.assertTrue(
        this.mitNullListe.durchschnitt() == 0
        , this.mitNullListe + " hat Durchschnitt 0"
            + " berechnet wurde " + this.mitNullListe.durchschnitt());
    Assertions.assertTrue(this.messwerteUnveraendert(this.mitNullListe)
        , "urspruengliche Werte " + this.genutzt 
          + " sollten nicht geaendert werden:" + this.mitNullListe);        
  }
}
