import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import java.util.ArrayList;

public class MessreiheZweiGleicheTest {
  private Messreihe m33244;
  private Messreihe m53532;
  private Messreihe m2344;
  private Messreihe leer;
  private Messreihe mitNullListe;
  private ArrayList<Integer> genutzt;

  private void merkeGenutzteWerte(Messreihe m){
    if (m.getMesswerte() == null){
      return;
    }
    for (int w: m.getMesswerte()) { // in Messreihe keine null-Werte
      this.genutzt.add(w);
    }
  }
  
  private boolean messwerteUnveraendert(Messreihe m){
    if (m.getMesswerte() == null){
      return this.genutzt.size() == 0;
    }
    for(int i = 0; i < this.genutzt.size(); i++) {
      if (this.genutzt.get(i) != m.getMesswerte().get(i)){
        return false;
      }
    }
    return m.getMesswerte().size() == this.genutzt.size();
  }
  
  @BeforeEach
  public void setUp() {
    this.genutzt = new ArrayList<Integer>();
    this.m33244 = new Messreihe("Ort1");
    this.m33244.hinzufuegen(3);
    this.m33244.hinzufuegen(3);
    this.m33244.hinzufuegen(2);
    this.m33244.hinzufuegen(4);
    this.m33244.hinzufuegen(4);
    this.m53532 = new Messreihe("Ort2");
    this.m53532.hinzufuegen(5);
    this.m53532.hinzufuegen(3);
    this.m53532.hinzufuegen(5);
    this.m53532.hinzufuegen(3);
    this.m53532.hinzufuegen(2);
    this.m2344 = new Messreihe("Ort3");
    this.m2344.hinzufuegen(2);
    this.m2344.hinzufuegen(3);
    this.m2344.hinzufuegen(4);
    this.m2344.hinzufuegen(4);
    this.leer = new Messreihe("Ort4");
    this.mitNullListe = new Messreihe("Ort5");
    this.mitNullListe.setMesswerte(null);
  }

  @Test
  public void test01() {
    this.merkeGenutzteWerte(this.m33244);
    Assertions.assertTrue(
        this.m33244.zweiGleiche()
        , this.m33244 + " hat aber zwei Gleiche");
    Assertions.assertTrue(this.messwerteUnveraendert(this.m33244)
        , "urspruengliche Werte " + this.genutzt 
          + " sollten nicht geaendert werden:" + this.m33244);    
  }

  @Test
  public void test02() {
    this.merkeGenutzteWerte(this.m53532);
    Assertions.assertTrue(
        this.m53532.zweiGleiche()
        , this.m53532 + " hat aber zwei Gleiche"
            + " berechnet wurde " + this.m53532.zweiGleiche());
    Assertions.assertTrue(this.messwerteUnveraendert(this.m53532)
        , "urspruengliche Werte " + this.genutzt 
          + " sollten nicht geaendert werden:" + this.m53532);        
  }

  @Test
  public void test03() {
    this.merkeGenutzteWerte(this.m2344);
    Assertions.assertTrue(
        this.m2344.zweiGleiche()
        , this.m2344 + " hat aber zwei Gleiche"
            + " berechnet wurde " + this.m2344.zweiGleiche());
    Assertions.assertTrue(this.messwerteUnveraendert(this.m2344)
        , "urspruengliche Werte " + this.genutzt 
          + " sollten nicht geaendert werden:" + this.m2344);        
  }

  @Test
  public void test04() {
    this.merkeGenutzteWerte(this.leer);
    Assertions.assertTrue(
        !this.leer.zweiGleiche()
        , this.leer + " hat keine zwei Gleiche");
  }

  @Test
  public void test05() {
    this.merkeGenutzteWerte(this.mitNullListe);
    Assertions.assertTrue(
        !this.mitNullListe.zweiGleiche()
        , this.mitNullListe + " hat keine zwei Gleichen");
  }

  @Test
  public void test06() {
    Messreihe m = new Messreihe("Alles in einem Test");
    m.hinzufuegen(42);
    this.merkeGenutzteWerte(m);
    Assertions.assertTrue(
        !m.zweiGleiche()
        , m + " hat keine zwei Gleichen");
    Assertions.assertTrue(this.messwerteUnveraendert(m)
        , "urspruengliche Werte " + this.genutzt 
          + " sollten nicht geaendert werden:" + m);    
  }

  @Test
  public void test07() {
    Messreihe m = new Messreihe("Alles in einem Test");
    for (int i = 0; i < 10000; i++) {
      m.hinzufuegen(i);
    }
    Assertions.assertTrue(
        !m.zweiGleiche()
        , "Liste von 0 bis 9999 hat keine zwei Gleichen");  
  }
}
